package org.claros.commons.utility;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @author Umut Gkbayrak
 */
public class Formatter {

	private Formatter() {
		super();
	}

	private static String format(DecimalFormat df, Object number) {
		if (number == null) {
			return null;
		}
		number = Utility.checkDecimalFormat(number);

		DecimalFormatSymbols dfs = df.getDecimalFormatSymbols();
		dfs.setDecimalSeparator(',');
		dfs.setGroupingSeparator('.');
		df.setDecimalFormatSymbols(dfs);
		try {
			if (!number.equals("-")) {
				return df.format(Double.parseDouble(number.toString()));
			} else {
				return number.toString();
			}
		} catch (NumberFormatException e) {
			return number.toString();
		}
	}

	public static String formatDecimal(Object number, boolean groupingActive) {
		if (groupingActive) {
			return format(new DecimalFormat("#,##0.00"), number);
		} else {
			return format(new DecimalFormat("0.00"), number);
		}
	}

	public static String formatInteger(Object number, boolean groupingActive) {
		if (groupingActive) {
			return format(new DecimalFormat("#,##0"), number);
		} else {
			return format(new DecimalFormat("##0"), number);
		}
	}

	public static String formatSensitive(Object number, boolean groupingActive, int digits) {
		String digitStr = "";
		for (int i = 0; i < digits; i++) {
			digitStr += "0";
		}
		if (groupingActive) {
			return format(new DecimalFormat("#,##0." + digitStr), number);
		} else {
			return format(new DecimalFormat("0." + digitStr), number);
		}
	}

	public static String formatWithPattern(Object number, String pattern) {
		return format(new DecimalFormat(pattern), number);
	}

	public static String formatDate(Object date, String pattern) {
		if (date != null && date instanceof Date) {
			SimpleDateFormat sdf = new SimpleDateFormat(pattern);
			return sdf.format(date);
		}
		return null;
	}

	public static double getDouble(Object obj) {
		if (obj == null) {
			return 0;
		}
		obj = Utility.checkDecimalFormat(obj);
		try {
			return Double.parseDouble(obj.toString());
		} catch (NumberFormatException e) {
			return 0;
		}
	}
}
