package org.claros.groupware.calendar.actions;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.claros.commons.configuration.PropertyFile;
import org.claros.commons.models.AuthProfile;
import org.claros.groupware.calendar.controllers.CalendarController;
import org.claros.groupware.common.actions.BaseLoggedAction;
import org.claros.groupware.preferences.controllers.UserPrefsController;
import org.claros.groupware.preferences.utility.UserPrefConstants;

/**
 * @version 	1.0
 * @author		Umut Gkbayrak
 */
public class ListMonthlyAction extends BaseLoggedAction {

	public ActionForward myExecute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception {
		AuthProfile auth = getAuthProfile(request);
		Calendar cal = getCalendar(auth, request);

		request.setAttribute("dateDisplayed", cal.getTime());

		// previous ve next calendar items
		Calendar prevCal = (Calendar)cal.clone();
		prevCal.add(Calendar.MONTH, -1);
		Calendar nextCal = (Calendar)cal.clone();
		nextCal.add(Calendar.MONTH, 1);
		request.setAttribute("prevTime", "" + prevCal.getTime().getTime());
		request.setAttribute("nextTime", "" + nextCal.getTime().getTime());

		ArrayList weeklyContent = CalendarController.populateMonthContent(auth, cal);
		request.setAttribute("monthlyContent", weeklyContent);
		
		request.getSession().setAttribute("calendarTab", "month");

		return mapping.findForward("/list_monthly.jsp");
	}

	/**
	 * URL parametresi olarak gelen t (time) var ise ondan bir calendar oluturur veya yoksa
	 * mevcut zaman dner. 
	 * @param request
	 * @return Calendar
	 */
	private Calendar getCalendar(AuthProfile auth, HttpServletRequest request) throws Exception {
		// Previous veya Next alanlarndan gelinmise parametre olarak bir t deikeni 
		// gelmi olmaldr. Bu t deikeni long olarak time deerini ierir. Bu deerden
		// bir calendar yaratp onun zerindne ilem yaplmaldr. Deilse o an zerinden
		// ilem yaplr. 
		HttpSession session = request.getSession();
		String time = (String)getVariable(request, "t");

		// get user timezone, if not available use server timezone
		String tz = UserPrefsController.getUserSetting(auth, UserPrefConstants.timeZone);
		if (tz == null) {
			tz = PropertyFile.getConfiguration("/config/config.xml").getString("common-params.server-timezone");
			if (tz == null) {
				tz = "Europe/Istanbul";
			}
		}

		// instantiate the calendar object if "t" variable is set reset time to "t"
		Calendar cal = Calendar.getInstance(TimeZone.getTimeZone(tz));
		if (time != null) {
			session.setAttribute("t", time);
			try {
				long tm = Long.parseLong(time);
				Date tmpDt = new Date();
				tmpDt.setTime(tm);
				cal.setTime(tmpDt);
			} catch (Exception e) {
				// Do nothing sier
			}
		}
		return cal;
	}

}
