package org.claros.groupware.calendar.controllers;

import java.sql.Timestamp;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.claros.commons.models.AuthProfile;
import org.claros.groupware.calendar.models.CalendarDailyItems;
import org.claros.groupware.calendar.models.CalendarHourItems;
import org.claros.groupware.calendar.models.CalendarItem;
import org.claros.groupware.calendar.models.CalendarItemWrap;
import org.claros.groupware.calendar.models.CalendarWeeklyItems;
import org.claros.groupware.calendar.utility.Constants;
import org.claros.groupware.calendar.utility.Utility;
import org.claros.groupware.preferences.controllers.UserPrefsController;
import org.claros.groupware.preferences.utility.UserPrefConstants;

/**
 * @author Umut Gokbayrak
 */
public class CalendarController {
	private static Log log = LogFactory.getLog(CalendarController.class); 

	/**
	 * @param auth
	 * @param cal
	 * @return
	 */
	public static CalendarDailyItems populateDailyContent(AuthProfile auth, Calendar cal) throws Exception {
		CalendarDailyItems out = new CalendarDailyItems();
		ArrayList apps = getAppointments(auth, cal, Constants.DAY);

		CalendarHourItems tmp = null;
		Timestamp begin, end = null;

		// reading the hour the day starts. 
		String dayStarts = UserPrefsController.getUserSetting(auth, UserPrefConstants.dayStarts);
		if (dayStarts == null) {
			dayStarts = "8";
		}
		DecimalFormat df = new DecimalFormat("00");
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd " + df.format(new Integer(dayStarts)) + ":00:00.000");
		Timestamp beginDay = Timestamp.valueOf(sdf.format(cal.getTime()));

//		Timestamp beginDay = Utility.getLimitTimestampFromCalendar(cal, true);
		cal.setTime(new Date(beginDay.getTime()));

		// assume the day how many hours?
		String calendarHours = UserPrefsController.getUserSetting(auth, UserPrefConstants.calendarHour);
		if (calendarHours == null) {
			calendarHours = "12";
		}
		int iCalHours = Integer.valueOf(calendarHours).intValue();

		CalendarItemWrap app = null;
		for (int i=0;i<iCalHours;i++) {
			tmp = new CalendarHourItems();
			tmp.setHour(new Integer(cal.get(Calendar.HOUR_OF_DAY)));
			
			for (int j=0;j<apps.size();j++) {
				app = (CalendarItemWrap)apps.get(j);

				begin = Utility.getHourLimitTimestampFromCalendar(cal, true);
				end = Utility.getHourLimitTimestampFromCalendar(cal, false);

				if (app.getOccuringDate().equals(begin) || app.getOccuringDate().equals(end) || (app.getOccuringDate().before(end) && app.getOccuringDate().after(begin))) {
					tmp.addAppointment(app);
				}
			}
			out.addHour(tmp);
			cal.add(Calendar.HOUR_OF_DAY, 1);
		}
		return out;
	}

	/**
	 * Ekranda gstermek zere haftalk ierii hazrlayan metoddur. Gerekli tekrarlanan 
	 * objeleri roll ettirir ve geriye gstermeye hazr bir arraylist ve ierisinde 
	 * AgendaDailyItem objeleri dndrr. 
	 * @param cif
	 * @param cal
	 * @return
	 */
	public static ArrayList populateWeekContent(AuthProfile auth, Calendar cal) {
		ArrayList out = new ArrayList();
		ArrayList apps = getAppointments(auth, cal, Constants.WEEK);

		CalendarDailyItems tmp = null;
		Timestamp begin = null;
		Timestamp end = null;
		CalendarItemWrap app = null;
		for (int i=0;i<7;i++) {
			tmp = new CalendarDailyItems();
			tmp.setDate(new Timestamp(cal.getTime().getTime()));
			begin = Utility.getLimitTimestampFromCalendar(cal, true);
			end = Utility.getLimitTimestampFromCalendar(cal, false);

			for (int j=0;j<apps.size();j++) {
				app = (CalendarItemWrap)apps.get(j);
				if (app.getOccuringDate().before(end) && app.getOccuringDate().after(begin)) {
					tmp.addAppointment(app);
				}
			}
			out.add(tmp);
			cal.add(Calendar.DATE, 1);
		}
		return out;
	}

	/**
	 * Aylk olarak listelenecek kaytlarn listesini eker.  
	 * @param cif
	 * @param cal
	 * @return
	 */
	public static ArrayList populateMonthContent(AuthProfile auth, Calendar cal) {
		ArrayList out = new ArrayList();
		ArrayList apps = getAppointments(auth, cal, Constants.MONTH);

		// ilk haftann kontrol iin ilk haftann hangi gnden baladn
		// belirlemek ve ona gre o ayn ka hafta tuttuunu belirlemek gereklidir.
		cal.set(Calendar.DAY_OF_MONTH, 1);
		int dayOfWeek = cal.get(Calendar.DAY_OF_WEEK);
		
		int emptyDaysInFirstWeek = 0;
		Calendar tmpCal = (Calendar)cal.clone();
		while (dayOfWeek != Calendar.MONDAY) {
			tmpCal.add(Calendar.DATE, -1);
			emptyDaysInFirstWeek++;
			dayOfWeek = tmpCal.get(Calendar.DAY_OF_WEEK);
		}
		
		int daysInMonth = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
		int totalDays = emptyDaysInFirstWeek + daysInMonth;
		int weeks = (int)Math.ceil((double)totalDays / 7);
		int exactDays = weeks * 7;
//		int month = cal.get(Calendar.MONTH);

		cal.add(Calendar.DATE, - emptyDaysInFirstWeek);
		CalendarDailyItems day = null;
		CalendarWeeklyItems week = new CalendarWeeklyItems();
		int weekCounter = 0;
		CalendarItemWrap app = null;
		Timestamp begin = null;
		Timestamp end = null;
		for (int i=0;i<exactDays;i++) {
			day = new CalendarDailyItems();
			// ilgili gn o ayn iindeki bir gne aittir. O gne ait bir event
			// varsa gne eklenmelidir. 
			begin = Utility.getLimitTimestampFromCalendar(cal, true);
			end = Utility.getLimitTimestampFromCalendar(cal, false);

			for (int j=0;j<apps.size();j++) {
				app = (CalendarItemWrap)apps.get(j);
				if (app.getOccuringDate().before(end) && app.getOccuringDate().after(begin)) {
					day.addAppointment(app);
				}
			}

			day.setDate(new Timestamp(cal.getTime().getTime()));
			week.addDay(day);
			cal.add(Calendar.DATE, 1);
			weekCounter++;
			if (weekCounter >= 7) {
				week.setWeekInYear(cal.get(Calendar.WEEK_OF_YEAR) -1 );
				out.add(week);
				week = new CalendarWeeklyItems();
				weekCounter = 0;
			}
		}
		return out;
	}


	/**
	 * O kullancya ait tm appointmentlarn listesini DB'den eker. Ardndan 
	 * tekrarlama deikenlerinin deerine gre karar alarak occuringDate (gerekleen) 
	 * zaman deikenini set eder. Bu esnada tekrarlanan bir obje ise roll ilemini
	 * yapar ve occuring date deeri bu gelen zaman aralnda olacak ekilde ayarlanr.
	 * Geriye gsterime hazr olmasa da gelen zaman aralna ait tm data mevcuttur. 
	 * timeVariable denilen deer WEEK, MONTH, ALL deerlerinden birisi olabilir. 
	 * @param cif
	 * @param cal
	 * @return
	 */	
	private static ArrayList getAppointments(AuthProfile auth, Calendar cal, int timeLimit) {
		ArrayList out = new ArrayList();

		try {
			// Gelen timeLimit'e gre haftalk, aylk ya da tm kaytlarn listesini 
			// almamz gerekiyor. Bunun iin birer balang ve son timestamp deerine
			// ihtiyacmz var. Bunlar elde ediyoruz.
			ArrayList res = Utility.getBeginEndTimestamps(cal, timeLimit);
			Timestamp begin = (Timestamp)res.get(0);
			Timestamp end = (Timestamp)res.get(1);

			List allApps = CalendarDBController.getCalendarItemsByUser(auth);

			// allApps iine ekilmi olan tm appointmentlarn bu hafta gsterilmesi gerekenlerini ekiyoruz.
			CalendarItem tmpCal = null;
			CalendarItemWrap tmpApp = null;
			Timestamp tmpDate = null;
			for (int i=0; i<allApps.size(); i++) {
				tmpCal = (CalendarItem)allApps.get(i);
				tmpApp = new CalendarItemWrap(tmpCal);
				
				tmpDate = tmpApp.getRecordDate();
				
				switch (tmpApp.getRepeatType().intValue()) {
					case Constants.REPEAT_TYPE_ONCE :
						// Bir kez durumu. Bu haftann bandan sonuna kadar olan srenin iinde yer alp almad baklr.
						if (tmpDate.after(begin) && tmpDate.before(end)) {
							tmpApp.setOccuringDate(tmpDate);
							out.add(tmpApp.clone());
						}
						break;
					case Constants.REPEAT_TYPE_MONTH : 
						// Her ay ayn gn. 
						tmpApp = Utility.rollAppointment(tmpApp, Calendar.MONTH, 1, begin, end);
						if (tmpApp != null) {
							out.add(tmpApp.clone());
						}
						break;
					case Constants.REPEAT_TYPE_YEAR : 
						// Her yl ayn gn
						tmpApp = Utility.rollAppointment(tmpApp, Calendar.YEAR, 1, begin, end);
						if (tmpApp != null) {
							out.add(tmpApp.clone());
						}
						break;
					case Constants.REPEAT_TYPE_WEEK : 
						// Her hafta ayn gn
						tmpApp = Utility.rollAppointment(tmpApp, Calendar.WEEK_OF_YEAR, 1, begin, end);
						if (tmpApp != null) {
							out.add(tmpApp.clone());
							if (timeLimit == Constants.MONTH) {
								while (tmpApp != null && (tmpApp.getOccuringDate() == null || tmpApp.getOccuringDate().before(end))) {
									tmpApp = Utility.rollAppointmentOnce(tmpApp, Calendar.WEEK_OF_YEAR, 1, end);
									if (tmpApp != null) {
										out.add(tmpApp.clone());
									}
								}
							}
						}
						break;
					case Constants.REPEAT_TYPE_TWO_WEEK :
						// ki haftada bir ayn gn
						tmpApp = Utility.rollAppointment(tmpApp, Calendar.WEEK_OF_YEAR, 2, begin, end);
						if (tmpApp != null) {
							out.add(tmpApp.clone());
							if (timeLimit == Constants.MONTH) {
								while (tmpApp != null && (tmpApp.getOccuringDate() == null || tmpApp.getOccuringDate().before(end))) {
									tmpApp = Utility.rollAppointmentOnce(tmpApp, Calendar.WEEK_OF_YEAR, 2, end);
									if (tmpApp != null) {
										out.add(tmpApp.clone());
									}
								}
							}
						}
						break;
				}
			}
		} catch (Exception e) {
			log.error(e);
		}
		// ekranda gsterilecek olan kaytlar artk tarih srasna gre sort ediyoruz.
		// bu sralamann sonucunda predefined ve private kaytlar tek bir arraylist
		// ierisinde sorted olarak yer alacaktr.
		Collections.sort(out);

		return out;
	}

}
