package org.claros.groupware.calendar.controllers;

import java.util.List;

import org.claros.commons.exception.NoPermissionException;
import org.claros.commons.models.AuthProfile;
import org.claros.groupware.calendar.models.CalendarItem;
import org.claros.groupware.common.utility.Constants;
import org.claros.groupware.common.utility.Utility;

import com.jenkov.mrpersister.impl.mapping.AutoGeneratedColumnsMapper;
import com.jenkov.mrpersister.itf.IGenericDao;
import com.jenkov.mrpersister.itf.mapping.IObjectMappingKey;
import com.jenkov.mrpersister.util.JdbcUtil;

/**
 * @author Umut Gokbayrak
 */
public class CalendarDBController {
	
	public static List getCalendarItemsByUser(AuthProfile auth) throws Exception {
		IGenericDao dao = null;
		List items = null;
		try {
			dao = Utility.getDbConnection();
			String username = auth.getUsername();
			
			String sql = "select * from CALENDAR_ITEMS where user=?";

			items = dao.readList(CalendarItem.class, sql, new Object[] {username});
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
		return items;
	}

	/**
	 * @param auth
	 * @param item
	 */
	public static void saveEvent(AuthProfile auth, CalendarItem item) throws Exception {
		IGenericDao dao = null;
		try {
			dao = Utility.getDbConnection();
			
			Long id = item.getId();
			if (id == null) {
				// it is an insert
				IObjectMappingKey myObj = Constants.persistMan.getObjectMappingFactory().createInstance(CalendarItem.class, new AutoGeneratedColumnsMapper(true));
				dao.insert(myObj, item);
			} else {
				// it is an update
				String username = auth.getUsername();
				if (!item.getUser().equals(username)) {
					throw new NoPermissionException();
				}
				dao.update(item);
			}
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
	}

	public static void deleteEvent(AuthProfile auth, Long eventId) throws Exception {
		CalendarItem tmp = getEventById(auth, eventId);
		if (!tmp.getUser().equals(auth.getUsername())) {
			throw new NoPermissionException();
		}

		IGenericDao dao = null;
		try {
			dao = Utility.getDbConnection();
			dao.deleteByPrimaryKey(CalendarItem.class, eventId);
		} catch (Exception e) {
			// do nothing sier
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
	}

	public static CalendarItem getEventById(AuthProfile auth, Long eventId) throws Exception {
		IGenericDao dao = null;
		CalendarItem event = null;
		try {
			dao = Utility.getDbConnection();
			String username = auth.getUsername();
			
			String sql = "select * from CALENDAR_ITEMS where user=? and id = ?";
			event = (CalendarItem)dao.read(CalendarItem.class, sql, new Object[] {username, eventId});
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
		return event;
	}
}
