package org.claros.groupware.common.actions;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.claros.commons.exception.ClarosBaseException;
import org.claros.commons.mail.exception.ProtocolNotAvailableException;
import org.claros.commons.mail.exception.ServerDownException;
import org.claros.commons.mail.models.ConnectionMetaHandler;
import org.claros.commons.mail.models.ConnectionProfile;
import org.claros.commons.models.AuthProfile;
import org.claros.groupware.common.utility.Constants;
import org.claros.groupware.preferences.controllers.UserPrefsController;
import org.claros.groupware.preferences.utility.UserPrefConstants;
import org.claros.groupware.rss.controllers.NewsController;
import org.claros.groupware.webmail.controllers.FolderController;
import org.claros.groupware.webmail.factory.FolderControllerFactory;

/**
 * @author Umut Gokbayrak
 */
public abstract class BaseLoggedAction extends BaseClarosAction {
	public abstract ActionForward myExecute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception;
    
	/* (non-Javadoc)
	 * @see org.apache.struts.action.Action#execute(org.apache.struts.action.ActionMapping, org.apache.struts.action.ActionForm, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception {
		commonFunctions(response);

		ActionForward forward = null;
		try {
			AuthProfile auth = getAuthProfile(request);
			if (auth != null) {
				// rss feeds (don't worry about performance, they are cached.)
				List news = getRssFeeds(auth);
				if (news == null || news.size() == 0) {
					request.setAttribute("showNews" , new Boolean(false));
				} else {
					request.setAttribute("showNews" , new Boolean(true));
					request.setAttribute("news", news);
				}
				
				// get all folders.
				ConnectionMetaHandler handler = getConnectionHandler(request);
				ConnectionProfile profile = getConnectionProfile(request);

				FolderControllerFactory factory = new FolderControllerFactory(auth, profile, handler);
				FolderController foldCont = factory.getFolderController();

				List folders = foldCont.getFolders();
				request.setAttribute("mailFolders", folders);

				List notesFolders = org.claros.groupware.notes.controllers.FolderController.getFolders(auth);
				request.setAttribute("notesFolders", notesFolders);

				// if any valid from address and full name is not set, then redirect user to preferences page.
				String fullName = UserPrefsController.getUserSetting(auth, UserPrefConstants.fullName);
				String from = UserPrefsController.getUserSetting(auth, UserPrefConstants.fromAddress);
				
				if (fullName == null || from == null || fullName.trim().length() == 0 || from.trim().length() == 0) {
					if (request.getRequestURI().indexOf("/preferences/") < 0 && 
						request.getRequestURI().indexOf("/error/") < 0 && 
						request.getRequestURI().indexOf("/profiling/") < 0) {
						forward = mapping.findForward("/preferences/prefEntry.do");
					} else {
						forward = myExecute(mapping, form, request, response);
					}
				} else {
					forward = myExecute(mapping, form, request, response);
				}
			} else {
				request.setAttribute("sessionExpired", new Boolean(true));
				forward = mapping.findForward("/profiling/loginStart.do");
			}
		} catch (ProtocolNotAvailableException e) {
			request.setAttribute("myexception", e.getMessage());
			forward = mapping.findForward("/profiling/loginStart.do");
		} catch (ServerDownException e) {
			request.setAttribute("myexception", e.getMessage());
			forward = mapping.findForward("/profiling/loginStart.do");
		} catch (Exception e) {
			ClarosBaseException ce = new ClarosBaseException(e);
			ce.setUserMessage(e.getMessage());
			request.setAttribute("myexception", ce);
			forward = mapping.findForward(Constants.GENERALERR);
		}
		return forward;
	}

	/**
	 * 
	 */
	private List getRssFeeds(AuthProfile auth) throws Exception {
		String showNews = UserPrefsController.getUserSetting(auth, UserPrefConstants.showNews);
		if (showNews == null) {
			showNews = "yes";
		}
		
		if (showNews.equals("yes")) {
			return NewsController.getRssItems(auth);
		}
		return null;
	}

	public AuthProfile getAuthProfile(HttpServletRequest request) {
		return (AuthProfile)request.getSession().getAttribute("auth");
	}

	public ArrayList getUserSettings(HttpServletRequest request) {
		return (ArrayList)request.getSession().getAttribute("prefs");
	}
}
