package org.claros.groupware.common.filters;

import java.util.HashMap;
import java.util.Iterator;

import javax.servlet.http.HttpSessionEvent;
import javax.servlet.http.HttpSessionListener;

import org.claros.commons.cache.CacheManager;
import org.claros.commons.mail.models.ConnectionMetaHandler;
import org.claros.commons.mail.models.ConnectionProfile;
import org.claros.commons.mail.protocols.Protocol;
import org.claros.commons.mail.protocols.ProtocolFactory;
import org.claros.commons.models.AuthProfile;
import org.claros.taglibs.page.Paginator;
import org.claros.taglibs.page.PaginatorHandler;

/**
 * @version 	1.0
 * @author		Umut Gkbayrak
 */
public class SessionListener implements HttpSessionListener {
	private static int onlineUsersCount = 0;
	private static HashMap onlineUsers = new HashMap();

	/* (non-Javadoc)
	 * @see javax.servlet.http.HttpSessionListener#sessionCreated(javax.servlet.http.HttpSessionEvent)
	 */
	public void sessionCreated(HttpSessionEvent arg0) {
		onlineUsersCount++;
	}
	
	public static void addOnlineUser(String username, String sessionId) {
		onlineUsers.put(username, sessionId);
	}

	/* (non-Javadoc)
	 * @see javax.servlet.http.HttpSessionListener#sessionDestroyed(javax.servlet.http.HttpSessionEvent)
	 */
	public void sessionDestroyed(HttpSessionEvent arg0) {
		onlineUsersCount--;
		AuthProfile auth = (AuthProfile)arg0.getSession().getAttribute("auth");
		if (auth != null) {
			onlineUsers.remove(auth.getUsername());
		}

		ConnectionProfile con = (ConnectionProfile)arg0.getSession().getAttribute("profile");
		ConnectionMetaHandler handler = (ConnectionMetaHandler)arg0.getSession().getAttribute("handler");
		
		if (auth != null && con != null && handler != null) {
			ProtocolFactory factory = new ProtocolFactory(con, auth, handler);
			Protocol protocol = factory.getProtocol(null);
			protocol.disconnect();
		}
		
		// sayfalandrma cache'i de temizlenmeli.
		PaginatorHandler page = (PaginatorHandler)arg0.getSession().getAttribute("paginator");
		if (page != null) {
			Paginator.invalidateSession(page.getId());
		}
		
		// Cache'de birikmi data temizlenmeli. 
		if (auth != null) {
			CacheManager.invalidate(auth.getUsername() + "news");
		}
	}
	
	public static String getUserBySessionId(String sessionId) {
		Iterator iter = onlineUsers.keySet().iterator();
		String tmp = null;
		while (iter.hasNext()) {
			tmp = (String)iter.next();
			if (onlineUsers.get(tmp).equals(sessionId)) {
				return tmp;
			}
		}
		return null;
	}
}
