package org.claros.groupware.preferences.controllers;

import java.util.HashMap;
import java.util.List;

import org.claros.commons.models.AuthProfile;
import org.claros.groupware.common.utility.Constants;
import org.claros.groupware.common.utility.Utility;
import org.claros.groupware.preferences.models.UserSetting;

import com.jenkov.mrpersister.impl.mapping.AutoGeneratedColumnsMapper;
import com.jenkov.mrpersister.itf.IGenericDao;
import com.jenkov.mrpersister.itf.mapping.IObjectMappingKey;
import com.jenkov.mrpersister.util.JdbcUtil;

/**
 * @author Umut Gokbayrak
 */
public class UserPrefsController {
	public UserPrefsController() {
		super();
	}
	
	/**
	 * Fetches all user preferences from DB.
	 * @param auth
	 * @return
	 * @throws Exception
	 */
	public static HashMap getUserPreferences(AuthProfile auth) throws Exception {
		HashMap prefMap = new HashMap();
		IGenericDao dao = null;
		try {
			dao = Utility.getDbConnection();
			
			String username = auth.getUsername();
			String sql = "select * from USERSETTINGS where user=?";
			List prefs = dao.readList(UserSetting.class, sql, new Object[] {username});
			
			if (prefs != null) {
				UserSetting tmp = null;
				for (int i=0;i<prefs.size();i++) {
					tmp = (UserSetting)prefs.get(i);
					prefMap.put(tmp.getKeyword(), tmp.getValue());
				}
			}
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
		return prefMap;
	}

	/**
	 * Retrieves a single preference setting from DB.
	 * @param auth
	 * @param key
	 * @return
	 * @throws Exception
	 */
	public static String getUserSetting(AuthProfile auth, String key) throws Exception {
		HashMap map = getUserPreferences(auth);
		return (String)map.get(key);
	}

	/**
	 * Saves a preference setting row into DB.
	 * @param auth
	 * @param key
	 * @param value
	 * @throws Exception
	 */
	public static void saveItem(AuthProfile auth, String key, String value) throws Exception {
		IGenericDao dao = null;
		try {
			dao = Utility.getDbConnection();
			
			String username = auth.getUsername();
			String sql = "select * from USERSETTINGS where user=? and keyword = ?";
			UserSetting pref = (UserSetting)dao.read(UserSetting.class, sql, new Object[] {username, key});
			if (pref == null) {
				// it is an insert
				pref = new UserSetting();
				pref.setKeyword((key == null) ? "" : key);
				pref.setUser((username == null) ? "" : username);
				pref.setValue((value == null) ? "" : value);
				
				IObjectMappingKey myObj = Constants.persistMan.getObjectMappingFactory().createInstance(UserSetting.class, new AutoGeneratedColumnsMapper(true));
				
				dao.insert(myObj, pref);
			} else {
				// it is an update
				pref.setKeyword(key);
				pref.setUser(username);
				pref.setValue(value);
				dao.update(pref);
			}
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
	}
}
