package org.claros.groupware.rss.controllers;

import java.net.URL;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.claros.commons.cache.Cache;
import org.claros.commons.cache.CacheManager;
import org.claros.commons.models.AuthProfile;
import org.claros.groupware.preferences.controllers.UserPrefsController;
import org.claros.groupware.preferences.utility.UserPrefConstants;
import org.claros.groupware.rss.models.NewsItem;
import org.gnu.stealthp.rsslib.RSSChannel;
import org.gnu.stealthp.rsslib.RSSException;
import org.gnu.stealthp.rsslib.RSSHandler;
import org.gnu.stealthp.rsslib.RSSItem;
import org.gnu.stealthp.rsslib.RSSParser;

/**
 * @author Umut Gokbayrak
 */
public class NewsController {
	private static Log log = LogFactory.getLog(NewsController.class);
	
	/**
	 * Rss reader
	 * @return
	 */
	private static List fetchRssItems(AuthProfile auth) {
		ArrayList allNews = new ArrayList();
		try {
			String newsUrl = UserPrefsController.getUserSetting(auth, UserPrefConstants.newsUrl);
			if (newsUrl == null) {
				newsUrl = "http://rss.cnn.com/rss/cnn_topstories.rss";
			}
			RSSHandler hand = new RSSHandler();
			try {
				URL u = new URL(newsUrl);
				RSSParser.parseXmlFile(u, hand, false);
			} catch(RSSException e) {
				e.printStackTrace();
			}
			RSSChannel ch = hand.getRSSChannel();
			
			
			String channelDescription = ch.getDescription();
			String channelUrl = ch.getLink();
			String channelTitle = ch.getTitle();
			
			LinkedList lst = hand.getRSSChannel().getItems();
			
			for(int i = 0; i < lst.size(); i++) {
				RSSItem itm = (RSSItem)lst.get(i);
				try {
					NewsItem news = new NewsItem();
					news.setChannelDescription(channelDescription);
					news.setChannelTitle(channelTitle);
					news.setChannelUrl(channelUrl);
					news.setTitle(itm.getTitle().replace('\n', ' '));
					news.setDate(itm.getDate());
					news.setDescription(itm.getDescription().replace('\n', ' '));
					news.setLink(itm.getLink());
					allNews.add(news);
				} catch (Exception e1) {
					log.debug("Unable to add news item: " + itm.toString(), e1);
				}
			}
		} catch (Exception e) {
			log.debug("RSS Feed cannot be fetched and parsed.", e);
		}
		return allNews;
	}

	/**
	 * RSS feeds are cached...
	 * 
	 * @param user
	 * @return
	 */
	public static List getRssItems(AuthProfile auth) {
		String key = auth.getUsername() + "news";
		Cache c = CacheManager.getContent(key);
		List result = null;
		if (c == null || c.isExpired()) {
			long ttl = 1000 * 60 * 5;
			result = fetchRssItems(auth);
			CacheManager.putContent(key, result, ttl);
		} else {
			result = (List)c.getValue();
		}
		return result;
	}	
}
