package org.claros.groupware.webmail.actions;

import java.io.ByteArrayOutputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;

import javax.mail.Address;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.codec.binary.Hex;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.claros.commons.configuration.PropertyFile;
import org.claros.commons.exception.SystemException;
import org.claros.commons.mail.models.ConnectionMetaHandler;
import org.claros.commons.mail.models.ConnectionProfile;
import org.claros.commons.mail.models.Email;
import org.claros.commons.mail.models.EmailHeader;
import org.claros.commons.mail.models.EmailPart;
import org.claros.commons.mail.protocols.Smtp;
import org.claros.commons.mail.utility.Utility;
import org.claros.commons.models.AuthProfile;
import org.claros.groupware.common.actions.BaseLoggedAction;
import org.claros.groupware.contacts.controllers.ContactsController;
import org.claros.groupware.contacts.controllers.GroupsController;
import org.claros.groupware.contacts.models.Contact;
import org.claros.groupware.contacts.models.ContactGroup;
import org.claros.groupware.contacts.models.ContactGroupItem;
import org.claros.groupware.preferences.controllers.UserPrefsController;
import org.claros.groupware.preferences.utility.UserPrefConstants;
import org.claros.groupware.webmail.controllers.FolderController;
import org.claros.groupware.webmail.controllers.MailController;
import org.claros.groupware.webmail.factory.FolderControllerFactory;
import org.claros.groupware.webmail.factory.MailControllerFactory;
import org.claros.groupware.webmail.models.ComposeFormBean;
import org.claros.groupware.webmail.models.EmailDbItem;
import org.claros.groupware.webmail.models.FolderDbItem;
import org.hsqldb.lib.MD5;

/**
 * @version 	1.0
 * @author		Umut Gkbayrak
 */
public class ComposeEndAction extends BaseLoggedAction {

	public ActionForward myExecute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception {
		Smtp smtp = new Smtp(getConnectionProfile(request), getAuthProfile(request));
		ComposeFormBean bean = (ComposeFormBean)request.getSession().getAttribute("ComposeFormBean");

		// learn the global charset setting.
		String charset = PropertyFile.getConfiguration("/config/config.xml").getString("common-params.charset");

		// learn user preferences from the DB.
		AuthProfile auth = getAuthProfile(request);
		HashMap prefMap = UserPrefsController.getUserPreferences(auth);
		String fullName = (String)prefMap.get(UserPrefConstants.fullName);
		String replyTo = (String)prefMap.get(UserPrefConstants.replyToAddress);
		String from = (String)prefMap.get(UserPrefConstants.fromAddress);

		// now create a new email object.
		Email email = new Email();
		EmailHeader header = new EmailHeader();
		if (fullName != null) {
			header.setFrom(new Address[] {new InternetAddress(from, fullName, charset)});
		} else {
			header.setFrom(new Address[] {new InternetAddress(from)});
		}
		header.setTo(group2Addr(auth, nick2Addr(auth, Utility.stringToAddressArray(bean.getTo()))));
		header.setCc(group2Addr(auth, nick2Addr(auth, Utility.stringToAddressArray(bean.getCc()))));
		header.setBcc(group2Addr(auth, nick2Addr(auth, Utility.stringToAddressArray(bean.getBcc()))));
		header.setSubject(bean.getSubject());
		header.setDate(new Date());
		if (replyTo != null && replyTo.trim().length() != 0) {
			header.setFrom(new Address[] {new InternetAddress(replyTo)});
		}
		email.setBaseHeader(header);
		
		String messageBody = bean.getMessageBody();
		ArrayList parts = new ArrayList();
		EmailPart bodyPart = new EmailPart();
		if (bean.getHtmlEmail().equals("true")) {
			bodyPart.setContentType("text/html; charset=" + charset);
		} else {
			bodyPart.setContentType("text/plain; charset=" + charset);
		}
		bodyPart.setContent(messageBody);
		parts.add(0, bodyPart);
		
		// attach some files...
		ArrayList attachments = (ArrayList)request.getSession().getAttribute("attachments");
		if (attachments != null) {
			parts.addAll(attachments);
		}
		email.setParts(parts);
		
		// it is time to send the email object message
		HashMap sendRes = smtp.send(email);
		MimeMessage msg = (MimeMessage)sendRes.get("msg");
		
		// if we fail to send the message to any of the recepients
		// we should make a report about it to the user.
		Address[] sent = (Address[])sendRes.get("sent");
		Address[] invalid = (Address[])sendRes.get("invalid");
		Address[] fail = (Address[])sendRes.get("fail");
		if (sent != null) {
			request.setAttribute("sent", sent);
		}
		if (invalid != null) {
			request.setAttribute("invalid", invalid);
		}
		if (fail != null) {
			request.setAttribute("fail", fail);
		}
		
		// if save to sent items enabled, save the sent mail.
		String saveEnabled = UserPrefsController.getUserSetting(auth, UserPrefConstants.saveSent);
		if (saveEnabled == null || saveEnabled.equals("yes")) {
			saveSentMail(auth, msg, request);
		}
		
		// time to do cleaning
		request.getSession().setAttribute("ComposeFormBean", null);
		
		return mapping.findForward("/compose_result.jsp");
	}

	private void saveSentMail(AuthProfile auth, MimeMessage msg, HttpServletRequest request) throws Exception {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		msg.writeTo(bos);
		byte bMsg[] = bos.toByteArray();
					
		// serialize the message byte array
		ObjectOutputStream os = new ObjectOutputStream(bos);
		os.writeObject(bMsg);

		// create an email db item
		EmailDbItem item = new EmailDbItem();
		item.setEmail(bMsg);
		String md5Header = new String(Hex.encodeHex(MD5.digestBytes(bMsg))).toUpperCase(new Locale("en", "US"));

		ConnectionMetaHandler handler = getConnectionHandler(request);
		ConnectionProfile profile = getConnectionProfile(request);

		FolderControllerFactory factory = new FolderControllerFactory(auth, profile, handler);
		FolderController foldCont = factory.getFolderController();
		FolderDbItem fItem = foldCont.getSentItems();

		item.setUniqueId(md5Header);
		item.setFolderId(fItem.getId());
		item.setUnread(new Boolean(false));
		item.setUser(auth.getUsername());
		item.setSize(new Long(bMsg.length));

		// save the email db item.
		MailControllerFactory mailFact = new MailControllerFactory(auth, profile, handler, fItem.getFolderName());
		MailController mailCont = mailFact.getMailController();
		mailCont.appendEmail(item);
	}
	
	/**
	 * replaces the nicknames with addresses
	 * @param auth
	 * @param addrs
	 * @return
	 * @throws SystemException
	 * @throws ConfigurationException
	 */
	private Address[] nick2Addr(AuthProfile auth, Address[] addrs) throws SystemException, ConfigurationException {
		if (addrs != null) {
			InternetAddress adr = null;
			String strAdr = null;
			Contact tmp = null;
			String charset = PropertyFile.getConfiguration("/config/config.xml").getString("common-params.charset");
			String fullName = null;
			for (int i=0;i<addrs.length;i++) {
				try {
					adr = (InternetAddress)addrs[i];
					strAdr = adr.getAddress();
					tmp = ContactsController.searchContactByNick(auth.getUsername(), strAdr);
					if (tmp != null) {
						fullName = tmp.getFirstName() + " " + tmp.getLastName();
						if (tmp.getMiddleName() != null) {
							fullName = tmp.getFirstName() + " " + tmp.getMiddleName() + " " + tmp.getLastName();
						}
						adr.setAddress(tmp.getEmailPrimary());
						adr.setPersonal(fullName, charset);
					}
				} catch (Exception e) {
					// do nothing sier
				}
			}
		}
		return addrs;
	}

	/**
	 * Convert groups to address arrays
	 * @param auth
	 * @param addrs
	 * @return
	 * @throws SystemException
	 * @throws ConfigurationException
	 */
	private Address[] group2Addr(AuthProfile auth, Address[] addrs) throws SystemException, ConfigurationException {
		Address outAdr[] = null;
		if (addrs != null) {
			InternetAddress adr = null;
			String strAdr = null;
			String charset = PropertyFile.getConfiguration("/config/config.xml").getString("common-params.charset");
			String fullName = null;
			ArrayList newArr = new ArrayList();
			for (int i=0;i<addrs.length;i++) {
				try {
					adr = (InternetAddress)addrs[i];
					strAdr = adr.getAddress();
					ContactGroup group = GroupsController.searchByName(auth, strAdr);
					if (group != null) {
						List items = GroupsController.getGroupItemsByGroup(auth, group.getId());
						ContactGroupItem itm = null;
						Contact c = null;
						for (int j=0; j<items.size();j++) {
							itm = (ContactGroupItem)items.get(j);
							c = ContactsController.getContactById(auth, itm.getContactId());
							if (c.getEmailPrimary() != null && !c.getEmailPrimary().trim().equals("")) {
								InternetAddress myAdr = new InternetAddress();
								fullName = org.claros.groupware.contacts.utility.Utility.getFullName(c, true);
								myAdr.setPersonal(fullName, charset);
								myAdr.setAddress(c.getEmailPrimary());
								newArr.add(myAdr);
							}
						}
					} else {
						newArr.add(adr);
					}
				} catch (Exception e) {
					// do nothing sier
				}
			}
			
			outAdr = new Address[newArr.size()];
			Address a = null;
			for (int i=0; i<newArr.size();i++) {
				a = (Address)newArr.get(i);
				outAdr[i] = a;
			}
		}
		return outAdr;
	}

}
