package org.claros.groupware.webmail.actions;

import java.util.ArrayList;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.claros.commons.mail.models.ConnectionMetaHandler;
import org.claros.commons.mail.models.ConnectionProfile;
import org.claros.commons.mail.models.Email;
import org.claros.commons.mail.models.EmailPart;
import org.claros.commons.mail.parser.HTMLMessageParser;
import org.claros.commons.mail.utility.Constants;
import org.claros.commons.models.AuthProfile;
import org.claros.commons.utility.Utility;
import org.claros.groupware.common.actions.BaseLoggedAction;
import org.claros.groupware.preferences.controllers.UserPrefsController;
import org.claros.groupware.preferences.utility.UserPrefConstants;
import org.claros.groupware.webmail.controllers.MailController;
import org.claros.groupware.webmail.factory.MailControllerFactory;

/**
 * @author Umut Gokbayrak
 */
public class ReadMailAction extends BaseLoggedAction {

	/* (non-Javadoc)
	 * @see org.claros.groupware.common.actions.BaseClarosAction#myExecute(org.apache.struts.action.ActionMapping, org.apache.struts.action.ActionForm, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	public ActionForward myExecute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception {
		ActionForward forward = null;
		String sMsg = (String)getVariable(request, "msg");
		String folder = (String)getVariable(request, "fid");
		Long iMsg = null;
		try {
			iMsg = new Long(sMsg);
		} catch (NumberFormatException ne) {
			// do nothing sier
		}
		if (iMsg != null) {
			AuthProfile auth = getAuthProfile(request);
			ConnectionMetaHandler handler = getConnectionHandler(request);
			ConnectionProfile profile = getConnectionProfile(request);

			MailControllerFactory factory = new MailControllerFactory(auth, profile, handler, folder);
			MailController mailCont = factory.getMailController();
			
			Email email = mailCont.getEmailById(iMsg);
			
			// learn if the user wants to view HTML e-mail or not. 
			// Sometimes users might prefer text messages rather than HTML ones
			// because of security concerns.
			String format = UserPrefsController.getUserSetting(getAuthProfile(request), UserPrefConstants.mailFormat);
			int i = -1;
			if (format == null || format.equals("html")) {
				i = findHtmlBody(email.getParts());
			}
			if (i == -1) {
				i = findTextBody(email.getParts());
			}

			String bodyText = new String();
			if (i != -1) {
				EmailPart body = (EmailPart)email.getParts().get(i);
				
				String cType = body.getContentType();
				if (cType.toLowerCase().startsWith("text/html")) {
					String bodyContent = (String)body.getContent();
					if (bodyContent.indexOf("") >= 0) {
						bodyContent = bodyContent.substring(0, bodyContent.indexOf(""));
					}
					
					String tmp = HTMLMessageParser.prepareInlineHTMLContent(email, bodyContent);
					bodyText += tmp;
				} else if (cType.toLowerCase().startsWith("text/plain")) {
					request.getSession().setAttribute("showDefaultCss", new Boolean(true));

					String bodyContent = "No body content to display.";
					if (body.getContent() instanceof String) {
						bodyContent = (String)body.getContent();
						if (bodyContent.indexOf("") >= 0) {
							bodyContent = bodyContent.substring(0, bodyContent.indexOf(""));
						}
					}
					String tmp = Utility.replaceAllOccurances(bodyContent, "\n", "<br>");
					bodyText += tmp;
				}
			} else {
				bodyText += "No body content to display";
			}

			if (bodyText.length() > 0) {
				email.setBodyText(bodyText);
			}
			if (profile.getProtocol().equals(Constants.POP3)) {
				mailCont.markAsRead(iMsg);
			}
			forward = mapping.findForward("/read_message.jsp");
			request.getSession().setAttribute("email", email);
		} else {
			forward = mapping.findForward("/mailbox.do");
		}
		return forward;
	}

	/**
	 * @param list
	 * @return
	 */
	private int findHtmlBody(ArrayList parts) {
		for (int i=0;i<parts.size();i++) {
			EmailPart body = (EmailPart)parts.get(i);
			String cType = body.getContentType();
			if (cType.toLowerCase().startsWith("text/html")) {
				return i;
			}
		}
		return -1;
	}

	private int findTextBody(ArrayList parts) {
		for (int i=0;i<parts.size();i++) {
			EmailPart body = (EmailPart)parts.get(i);
			String cType = body.getContentType();
			if (cType.toLowerCase().startsWith("text/plain")) {
				return i;
			}
		}
		return -1;
	}

}
